#pragma once

#include <thread>
#include <functional>
#include <stdexcept>
#include <condition_variable>
#include <mutex>
#include <utility>
#include <MultiThreading/Task.h>

namespace MultiThreading {
    class Thread;
}

class MultiThreading::Thread {
private:
    std::shared_ptr<TaskBase> current_task = nullptr;
    std::atomic<bool> stop = false;
    std::condition_variable cv;
    std::thread worker;
    std::mutex mtx;
    bool busy = false;
private:
    void Runner();
public:
    [[nodiscard]] bool SetTask(const std::function<void()>& task);
    [[nodiscard]] bool SetTask(std::shared_ptr<TaskBase> task);
    [[nodiscard]] bool Busy() const { return busy; }
    void Join() { if (worker.joinable()) worker.join(); };
public:
    Thread() { worker = std::thread([this] { this->Runner(); }); }
    explicit Thread(std::shared_ptr<TaskBase> task);
    explicit Thread(const std::function<void()>& task);
    ~Thread();
};