#pragma once

#include <MultiThreading/Thread.h>
#include <vector>
#include <queue>
namespace MultiThreading {
    class ThreadPool;
}

class MultiThreading::ThreadPool {
private:
    std::vector<MultiThreading::Thread*> threads;
    std::queue<std::shared_ptr<TaskBase>> queue;
    std::mutex queue_mutex;
private:
    /// @returns nullptr if the queue is empty.
    std::shared_ptr<TaskBase> Dequeue();
    void Runner(const std::shared_ptr<TaskBase>& task);
public:
    void Enqueue(const std::shared_ptr<MultiThreading::TaskBase>& task);
    void Enqueue(const std::function<void()>& task);
public:
    [[nodiscard]] unsigned int ThreadCount() const { return threads.size(); }
    [[nodiscard]] unsigned int PendingTasks();
    /// @returns Whether a task you enqueue would have to wait.
    [[nodiscard]] bool Busy();
public:
    explicit ThreadPool(unsigned int thread_count = std::thread::hardware_concurrency());
    ~ThreadPool();
};